@@ -7,6 +7,8 @@ import time
 import requests
 import yaml
 from requests.exceptions import HTTPError
+from requests.packages.urllib3.util import Retry
+from requests.adapters import HTTPAdapter
 
 import gitlab_watchman.config as cfg
 import gitlab_watchman.logger as logger
@@ -19,7 +21,7 @@ class GitLabAPIClient(object):
         self.base_url = base_url.rstrip('\\')
         self.per_page = 100
         self.session = session = requests.session()
-        session.mount(self.base_url, requests.adapters.HTTPAdapter())
+        session.mount(self.base_url, HTTPAdapter(max_retries=Retry(connect=3, backoff_factor=1)))
         session.headers.update({'Authorization': 'Bearer {}'.format(self.token)})
 
     def make_request(self, url, params=None, data=None, method='GET', verify_ssl=True):
@@ -166,12 +168,12 @@ def convert_time(timestamp):
 def deduplicate(input_list):
     """Removes duplicates where results are returned by multiple queries"""
 
-    list_of_strings = [json.dumps(d, sort_keys=True) for d in input_list]
+    list_of_strings = [json.dumps(d) for d in input_list]
     list_of_strings = set(list_of_strings)
     return [json.loads(s) for s in list_of_strings]
 
 
-def search_commits(gitlab: GitLabAPIClient, log_handler, search_terms, regex, timeframe=cfg.ALL_TIME):
+def search_commits(gitlab: GitLabAPIClient, log_handler, rule, timeframe=cfg.ALL_TIME):
     """Uses the Search API to get commits matching a search term.
         This is then filtered by regex to find true matches"""
 
@@ -182,11 +184,11 @@ def search_commits(gitlab: GitLabAPIClient, log_handler, search_terms, regex, ti
     else:
         print = builtins.print
 
-    for query in search_terms:
+    for query in rule.get('strings'):
         commit_list = gitlab.global_search('search', query, search_scope='commits')
         print('{} commits found matching: {}'.format(len(commit_list), query.replace('"', '')))
         for commit in commit_list:
-            r = re.compile(regex)
+            r = re.compile(rule.get('pattern'))
             if convert_time(commit.get('committed_date')) > (now - timeframe) and r.search(str(commit.get('message'))):
                 project = gitlab.get_project(commit.get('project_id'))
                 results_dict = {
@@ -196,9 +198,20 @@ def search_commits(gitlab: GitLabAPIClient, log_handler, search_terms, regex, ti
                     'committed_date': commit.get('committed_date'),
                     'committer_name': commit.get('committer_name'),
                     'committer_email': commit.get('committer_email'),
-                    'project_url': project.get('web_url'),
-                    'project_id': commit.get('project_id'),
-                    'project_name': project.get('name')
+                    'match_string': r.search(str(commit.get('message'))).group(0),
+                    'project': {
+                        'project_url': project.get('web_url'),
+                        'project_id': commit.get('project_id'),
+                        'project_name': project.get('name'),
+                        'namespace': {
+                            'namespace_id': project.get('namespace').get('id'),
+                            'name': project.get('namespace').get('name'),
+                            'kind': project.get('namespace').get('kind'),
+                            'full_path': project.get('namespace').get('full_path'),
+                            'parent_id': project.get('namespace').get('parent_id'),
+                            'web_url': project.get('namespace').get('web_url')
+                        }
+                    }
                 }
 
                 results.append(results_dict)
@@ -210,7 +223,7 @@ def search_commits(gitlab: GitLabAPIClient, log_handler, search_terms, regex, ti
         print('No matches found after filtering')
 
 
-def search_milestones(gitlab: GitLabAPIClient, log_handler, search_terms, regex, timeframe=cfg.ALL_TIME):
+def search_milestones(gitlab: GitLabAPIClient, log_handler, rule, timeframe=cfg.ALL_TIME):
     """Uses the Search API to get milestones matching a search term.
          This is then filtered by regex to find true matches"""
 
@@ -221,25 +234,36 @@ def search_milestones(gitlab: GitLabAPIClient, log_handler, search_terms, regex,
     else:
         print = builtins.print
 
-    for query in search_terms:
+    for query in rule.get('strings'):
         milestone_list = gitlab.global_search('search', query, search_scope='milestones')
         print('{} milestones found matching: {}'.format(len(milestone_list), query.replace('"', '')))
         for milestone in milestone_list:
-            r = re.compile(regex)
+            r = re.compile(rule.get('pattern'))
             if convert_time(milestone.get('updated_at')) > (now - timeframe) and r.search(
-                    str(milestone['description'])):
+                    str(milestone.get('description'))):
                 project = gitlab.get_project(milestone.get('project_id'))
                 results_dict = {
-                    "milestone_id": milestone.get('id'),
-                    "title": milestone.get('title'),
-                    "description": milestone.get('decription'),
-                    "created_at": milestone.get('created_at'),
-                    "updated_at": milestone.get('updated_at'),
-                    "due_date": milestone.get('due_date'),
-                    "start_date": milestone.get('start_date'),
-                    "project_url": project.get('web_url'),
-                    "project_id": milestone.get('project_id'),
-                    "project_name": project.get('name')
+                    'milestone_id': milestone.get('id'),
+                    'title': milestone.get('title'),
+                    'description': milestone.get('description'),
+                    'created_at': milestone.get('created_at'),
+                    'updated_at': milestone.get('updated_at'),
+                    'due_date': milestone.get('due_date'),
+                    'start_date': milestone.get('start_date'),
+                    'match_string': r.search(str(milestone.get('description'))).group(0),
+                    'project': {
+                        'project_url': project.get('web_url'),
+                        'project_id': milestone.get('project_id'),
+                        'project_name': project.get('name'),
+                        'namespace': {
+                            'namespace_id': project.get('namespace').get('id'),
+                            'name': project.get('namespace').get('name'),
+                            'kind': project.get('namespace').get('kind'),
+                            'full_path': project.get('namespace').get('full_path'),
+                            'parent_id': project.get('namespace').get('parent_id'),
+                            'web_url': project.get('namespace').get('web_url')
+                        }
+                    }
                 }
 
                 results.append(results_dict)
@@ -251,7 +275,7 @@ def search_milestones(gitlab: GitLabAPIClient, log_handler, search_terms, regex,
         print('No matches found after filtering')
 
 
-def search_issues(gitlab: GitLabAPIClient, log_handler, search_terms, regex, timeframe=cfg.ALL_TIME):
+def search_issues(gitlab: GitLabAPIClient, log_handler, rule, timeframe=cfg.ALL_TIME):
     """Uses the Search API to get issues matching a search term.
          This is then filtered by regex to find true matches"""
 
@@ -262,11 +286,11 @@ def search_issues(gitlab: GitLabAPIClient, log_handler, search_terms, regex, tim
     else:
         print = builtins.print
 
-    for query in search_terms:
+    for query in rule.get('strings'):
         issue_list = gitlab.global_search('search', query, search_scope='issues')
         print('{} issues found matching: {}'.format(len(issue_list), query.replace('"', '')))
         for issue in issue_list:
-            r = re.compile(regex)
+            r = re.compile(rule.get('pattern'))
             if convert_time(issue.get('updated_at')) > (now - timeframe) and r.search(str(issue.get('description'))):
                 project = gitlab.get_project(issue.get('project_id'))
                 results_dict = {
@@ -282,11 +306,20 @@ def search_issues(gitlab: GitLabAPIClient, log_handler, search_terms, regex, tim
                     'author_username': issue.get('author').get('username'),
                     'due_date': issue.get('due_date'),
                     'confidential': issue.get('confidential'),
-                    'project_url': project.get('web_url'),
-                    'project_id': issue.get('project_id'),
-                    'project_name': project.get('name'),
-                    'assignee_id': '',
-                    'assignee_username': ''
+                    'match_string': r.search(str(issue.get('description'))).group(0),
+                    'project': {
+                        'project_url': project.get('web_url'),
+                        'project_id': issue.get('project_id'),
+                        'project_name': project.get('name'),
+                        'namespace': {
+                            'namespace_id': project.get('namespace').get('id'),
+                            'name': project.get('namespace').get('name'),
+                            'kind': project.get('namespace').get('kind'),
+                            'full_path': project.get('namespace').get('full_path'),
+                            'parent_id': project.get('namespace').get('parent_id'),
+                            'web_url': project.get('namespace').get('web_url')
+                        }
+                    }
                 }
                 if issue.get('assignee'):
                     results_dict['assignee_id'] = issue.get('assignee').get('id')
@@ -301,7 +334,7 @@ def search_issues(gitlab: GitLabAPIClient, log_handler, search_terms, regex, tim
         print('No matches found after filtering')
 
 
-def search_wiki_blobs(gitlab: GitLabAPIClient, log_handler, search_terms, regex, timeframe=cfg.ALL_TIME):
+def search_wiki_blobs(gitlab: GitLabAPIClient, log_handler, rule, timeframe=cfg.ALL_TIME):
     """Uses the Search API to get wiki blobs matching a search term.
          This is then filtered by regex to find true matches"""
 
@@ -312,11 +345,11 @@ def search_wiki_blobs(gitlab: GitLabAPIClient, log_handler, search_terms, regex,
     else:
         print = builtins.print
 
-    for query in search_terms:
+    for query in rule.get('strings'):
         blob_list = gitlab.global_search('search', query, search_scope='wiki_blobs')
         print('{} wiki blobs found matching: {}'.format(len(blob_list), query.replace('"', '')))
         for blob in blob_list:
-            r = re.compile(regex)
+            r = re.compile(rule.get('pattern'))
             project = gitlab.get_project(blob.get('project_id'))
             if convert_time(project.get('last_activity_at')) > (now - timeframe) and r.search(str(blob.get('data'))):
                 results_dict = {
@@ -324,9 +357,20 @@ def search_wiki_blobs(gitlab: GitLabAPIClient, log_handler, search_terms, regex,
                     'basename': blob.get('basename'),
                     'data': blob.get('data'),
                     'path': blob.get('path'),
-                    'project_url': project.get('web_url'),
-                    'project_id': blob.get('project_id'),
-                    'project_name': project.get('name')
+                    'match_string': r.search(str(blob.get('data'))).group(0),
+                    'project': {
+                        'project_url': project.get('web_url'),
+                        'project_id': blob.get('project_id'),
+                        'project_name': project.get('name'),
+                        'namespace': {
+                            'namespace_id': project.get('namespace').get('id'),
+                            'name': project.get('namespace').get('name'),
+                            'kind': project.get('namespace').get('kind'),
+                            'full_path': project.get('namespace').get('full_path'),
+                            'parent_id': project.get('namespace').get('parent_id'),
+                            'web_url': project.get('namespace').get('web_url')
+                        }
+                    }
                 }
 
                 results.append(results_dict)
@@ -338,7 +382,7 @@ def search_wiki_blobs(gitlab: GitLabAPIClient, log_handler, search_terms, regex,
         print('No matches found after filtering')
 
 
-def search_merge_requests(gitlab: GitLabAPIClient, log_handler, search_terms, regex, timeframe=cfg.ALL_TIME):
+def search_merge_requests(gitlab: GitLabAPIClient, log_handler, rule, timeframe=cfg.ALL_TIME):
     """Uses the Search API to get merge requests matching a search term.
          This is then filtered by regex to find true matches"""
 
@@ -349,14 +393,14 @@ def search_merge_requests(gitlab: GitLabAPIClient, log_handler, search_terms, re
     else:
         print = builtins.print
 
-    for query in search_terms:
+    for query in rule.get('strings'):
         merge_request_list = gitlab.global_search('search', query, search_scope='merge_requests')
         print('{} merge requests found matching: {}'.format(len(merge_request_list), query.replace('"', '')))
         for merge_request in merge_request_list:
-            r = re.compile(regex)
-            if convert_time(merge_request['updated_at']) > (now - timeframe) and \
-                    r.search(str(merge_request['description'])):
-                project = gitlab.get_project(merge_request['project_id'])
+            r = re.compile(rule.get('pattern'))
+            if convert_time(merge_request.get('updated_at')) > (now - timeframe) and \
+                    r.search(str(merge_request.get('description'))):
+                project = gitlab.get_project(merge_request.get('project_id'))
                 results_dict = {
                     'merge_request_id': merge_request.get('id'),
                     'title': merge_request.get('title'),
@@ -368,11 +412,20 @@ def search_merge_requests(gitlab: GitLabAPIClient, log_handler, search_terms, re
                     'author_username': merge_request.get('author').get('username'),
                     'merge_status': merge_request.get('merge_status'),
                     'url': merge_request.get('url'),
-                    'project_url': project.get('web_url'),
-                    'project_id': merge_request.get('id'),
-                    'project_name': project.get('name'),
-                    'assignee_id': '',
-                    'assignee_username': ''
+                    'match_string': r.search(str(merge_request.get('description'))).group(0),
+                    'project': {
+                        'project_url': project.get('web_url'),
+                        'project_id': merge_request.get('project_id'),
+                        'project_name': project.get('name'),
+                        'namespace': {
+                            'namespace_id': project.get('namespace').get('id'),
+                            'name': project.get('namespace').get('name'),
+                            'kind': project.get('namespace').get('kind'),
+                            'full_path': project.get('namespace').get('full_path'),
+                            'parent_id': project.get('namespace').get('parent_id'),
+                            'web_url': project.get('namespace').get('web_url')
+                        }
+                    }
                 }
                 if merge_request.get('assignee'):
                     results_dict['assignee_id'] = merge_request.get('assignee').get('id')
@@ -387,7 +440,7 @@ def search_merge_requests(gitlab: GitLabAPIClient, log_handler, search_terms, re
         print('No matches found after filtering')
 
 
-def search_blobs(gitlab: GitLabAPIClient, log_handler, search_terms, regex, timeframe=cfg.ALL_TIME):
+def search_blobs(gitlab: GitLabAPIClient, log_handler, rule, timeframe=cfg.ALL_TIME):
     """Uses the Search API to get blobs matching a search term.
          This is then filtered by regex to find true matches"""
 
@@ -398,21 +451,33 @@ def search_blobs(gitlab: GitLabAPIClient, log_handler, search_terms, regex, time
     else:
         print = builtins.print
 
-    for query in search_terms:
+    for query in rule.get('strings'):
         blob_list = gitlab.global_search('search', query, search_scope='blobs')
         print('{} blobs found matching search term: {}'.format(len(blob_list), query.replace('"', '')))
         for blob in blob_list:
-            r = re.compile(regex)
+            r = re.compile(rule.get('pattern'))
             project = gitlab.get_project(blob.get('project_id'))
-            if convert_time(project.get('last_activity_at')) > (now - timeframe) and r.search(str(blob)):
+            if convert_time(project.get('last_activity_at')) > (now - timeframe) and r.search(str(blob.get('data'))):
                 results_dict = {
                     'blob_id': blob.get('id'),
                     'basename': blob.get('basename'),
                     'data': blob.get('data'),
                     'path': blob.get('path'),
-                    'project_url': project.get('web_url'),
-                    'project_id': blob.get('project_id'),
-                    'project_name': project.get('name')
+                    'match_string': r.search(str(blob.get('data'))).group(0),
+                    'last_activity_at': blob.get('last_activity_at'),
+                    'project': {
+                        'project_url': project.get('web_url'),
+                        'project_id': blob.get('project_id'),
+                        'project_name': project.get('name'),
+                        'namespace': {
+                            'namespace_id': project.get('namespace').get('id'),
+                            'name': project.get('namespace').get('name'),
+                            'kind': project.get('namespace').get('kind'),
+                            'full_path': project.get('namespace').get('full_path'),
+                            'parent_id': project.get('namespace').get('parent_id'),
+                            'web_url': project.get('namespace').get('web_url')
+                        }
+                    }
                 }
 
                 results.append(results_dict)
@@ -423,62 +488,3 @@ def search_blobs(gitlab: GitLabAPIClient, log_handler, search_terms, regex, time
     else:
         print('No matches found after filtering')
 
-
-def get_public_variables(gitlab: GitLabAPIClient, log_handler, project_list):
-    """Searches all projects for public CICD variables"""
-
-    results = []
-    if isinstance(log_handler, logger.StdoutLogger):
-        print = log_handler.log_info
-    else:
-        print = builtins.print
-
-    for i in project_list:
-        if gitlab.get_variables(i.get('id')):
-            project = gitlab.get_project(i.get('id'))
-            results_dict = {
-                'project_id': project.get('id'),
-                'project_name': project.get('name'),
-                'repository_url': project.get('web_url'),
-                'last_activity': project.get('last_activity_at')
-            }
-            results.append(results_dict)
-    if results:
-        print('{} repositories with variables exposed'.format(len(results)))
-        return results
-    else:
-        print('No exposed variables found')
-
-# def search_notes(gitlab: GitLabAPIClient, search_terms, regex, project_id):
-#     """Searches the given project for any notes matching the search term.
-#         This is then filtered by regex to find true matches"""
-#
-#     for query in search_terms:
-#         results = []
-#         notes_list = gitlab.global_search('projects/{}/search'.format(project_id), query, search_scope='notes')
-#         for note in notes_list:
-#             r = re.compile(regex)
-#             if r.search(str(note.get('body'))):
-#                 project = gitlab.get_project(project_id)
-#                 print('Match found for {} in project {}'.format(query, project['name']))
-#                 results.append([project_id,
-#                                 project['name'],
-#                                 project['web_url'],
-#                                 note['body'],
-#                                 note['updated_at']])
-#         if results:
-#             print('{} matches found for {}'.format(len(results), query))
-#             return results
-
-# def write_notes(result_list, file_name):
-#     """Writes any found notes to CSV"""
-#
-#     headers = ['project_id',
-#                'project_name',
-#                'repository_url',
-#                'data',
-#                'last_activity']
-#     out_path = os.getcwd()
-#     path = '{}/{}.csv'.format(out_path, file_name)
-#     write_csv(headers, path, result_list)
-#     print('CSV written: {}'.format(path))
